Package: maptools
Version: 1.1-6
Date: 2022-12-12
Title: Tools for Handling Spatial Objects
Authors@R: c(
	person("Roger", "Bivand", role = c("cre", "aut"), email = "Roger.Bivand@nhh.no", comment=c(ORCID="0000-0003-2392-6140")),
	person("Nicholas", "Lewin-Koh", role = "aut"),
	person("Edzer", "Pebesma", role = "ctb"),
	person("Eric", "Archer", role = "ctb"),
	person("Adrian", "Baddeley", role = "ctb"),
	person("Nick", "Bearman", role = "ctb"),
	person("Hans-Jörg", "Bibiko", role = "ctb"),
	person("Steven", "Brey", role = "ctb"),
	person("Jonathan", "Callahan", role = "ctb"),
	person("German", "Carrillo", role = "ctb"),
	person("Stéphane", "Dray", role = "ctb"),
	person("David", "Forrest", role = "ctb"),
	person("Michael", "Friendly", role = "ctb"),
	person("Patrick", "Giraudoux", role = "ctb"),
	person("Duncan", "Golicher", role = "ctb"),
	person("Virgilio", "Gómez Rubio", role = "ctb"),
	person("Patrick", "Hausmann", role = "ctb"),
	person("Karl Ove", "Hufthammer", role = "ctb"),
	person("Thomas", "Jagger", role = "ctb"),
	person("Kent", "Johnson", role = "ctb"),
        person("Matthew", "Lewis", role = "ctb", comment=c(ORCID="0000-0003-2244-4078")),
	person("Sebastian", "Luque", role = "ctb"),
	person("Don", "MacQueen", role = "ctb"),
	person("Andrew", "Niccolai", role = "ctb"),
	person("Edzer", "Pebesma", role = "ctb"),
	person("Oscar", "Perpiñán Lamigueiro", role = "ctb"),
        person("Ethan", "Plunkett", role = "ctb"),
	person("Ege", "Rubak", role = "ctb", comment=c(ORCID="0000-0002-6675-533X")),
	person("Tom", "Short", role = "ctb"),
	person("Greg", "Snow", role = "ctb"),
	person("Ben", "Stabler", role = "ctb"),
	person("Murray", "Stokely", role = "ctb"),
	person("Rolf", "Turner", role = "ctb"))
Encoding: UTF-8
Depends: R (>= 2.10), sp (>= 1.0-11)
Imports: foreign (>= 0.8), methods, grid, lattice, stats, utils,
        grDevices
Suggests: rgeos (>= 0.1-8), spatstat.geom (>= 1.65-0), PBSmapping,
        maps, RColorBrewer, raster, polyclip, plotrix, spatstat.linnet
        (>= 1.65-3), spatstat.utils (>= 1.19.0), spatstat (>= 2.0-0)
Enhances: gpclib
Description: Please note that 'maptools' will be retired during 2023, plan transition at your earliest convenience; some functionality will be moved to 'sp'. Set of tools for manipulating geographic data. It includes binary access to 'GSHHG' shoreline files. The package also provides interface wrappers for exchanging spatial objects with packages such as 'PBSmapping', 'spatstat.geom', 'maps', and others.
License: GPL (>= 2)
URL: http://maptools.r-forge.r-project.org/,
        https://r-forge.r-project.org/projects/maptools/
RoxygenNote: 6.1.1
NeedsCompilation: yes
Packaged: 2022-12-12 13:51:14 UTC; rsb
Author: Roger Bivand [cre, aut] (<https://orcid.org/0000-0003-2392-6140>),
  Nicholas Lewin-Koh [aut],
  Edzer Pebesma [ctb],
  Eric Archer [ctb],
  Adrian Baddeley [ctb],
  Nick Bearman [ctb],
  Hans-Jörg Bibiko [ctb],
  Steven Brey [ctb],
  Jonathan Callahan [ctb],
  German Carrillo [ctb],
  Stéphane Dray [ctb],
  David Forrest [ctb],
  Michael Friendly [ctb],
  Patrick Giraudoux [ctb],
  Duncan Golicher [ctb],
  Virgilio Gómez Rubio [ctb],
  Patrick Hausmann [ctb],
  Karl Ove Hufthammer [ctb],
  Thomas Jagger [ctb],
  Kent Johnson [ctb],
  Matthew Lewis [ctb] (<https://orcid.org/0000-0003-2244-4078>),
  Sebastian Luque [ctb],
  Don MacQueen [ctb],
  Andrew Niccolai [ctb],
  Edzer Pebesma [ctb],
  Oscar Perpiñán Lamigueiro [ctb],
  Ethan Plunkett [ctb],
  Ege Rubak [ctb] (<https://orcid.org/0000-0002-6675-533X>),
  Tom Short [ctb],
  Greg Snow [ctb],
  Ben Stabler [ctb],
  Murray Stokely [ctb],
  Rolf Turner [ctb]
Maintainer: Roger Bivand <Roger.Bivand@nhh.no>
Repository: CRAN
Date/Publication: 2022-12-14 23:30:03 UTC
