This file is intended to clarify ownership and copyright: where
possible individual files also carry brief copyright notices. 

Copyrights
==========

Files: src/shapefil.h, src/shpopen.c and src/shptree.c are
copyright (c) 1998 Frank Warmerdam, and released under the conditions
given in http://shapelib.maptools.org/license.html. His code is available
under the MIT or LGPL licence, and the latter is chosen here.

This software is available under the following "MIT Style" license, or
at the option of the licensee under the LGPL (see LICENSE.LGPL). This
option is discussed in more detail in shapelib.html.

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

Further details of Shapelib may be found at its website

http://shapelib.maptools.org/

Files: src/Rcentroid.c, src/Rshapeinfo.c, src/maptools.h, and
src/Rshapeget.c are copyright (c) 2000-2001 by Nicholas Lewin-Koh and are
subject to the licence at the foot of this file. Some portions of these
files are based on shapelib/contrib files, and copyright and license
conditions follow from the included code: RingCentroid_2d, shpcat.c,
shpinfo.c, and SHPRingDir_2d Copyright (c) 1999, Carl Anderson

Files: inst/shapes/baltim.*, inst/shapes/columbus.*, and inst/shapes/sids.* 
are derived by permission from SAL data files
prepared by Luc Anselin; they were downloaded from:

http://geodacenter.asu.edu/sdata

under the following conditions: "The data are provided as is, without 
any warranties. Please make sure to give proper credit when using 
them in any publications. Some of the data sets are purely illustrative 
and should not be used for substantive research."

Files: inst/shapes/fylk-val* are copyright Statens Kartverk 2001, and
were released (with a collection of other shapefiles) to the public domain
in July 2001. The link to the relevant page under http://www.statkart.no
has now expired, this copy downloaded 13 July 2001.

Files: inst/shapes/pointZ.* are a sample of points from a 3D point shapefile
kindly contributed by Michael Toews to be used here for illustrative purposes;
this sample is written out using writePointsShape(). 

File: inst/shapes/Testing.kml is contributed by Kent Johnson, released 
under GPL (>=2) as part of maptools.

File: data/wrld_simpl.rda is a modified SpatialPolygonsDataFrame object
generated from the TM_WORLD_BORDERS_SIMPL-0.2 shapefile downloaded from
http://mappinghacks.com/data/TM_WORLD_BORDERS_SIMPL-0.2.zip with the 
attribution: "The world_borders.zip dataset has been superseded by Bjørn 
Sandvik's improved version, TM_WORLD_BORDERS-0.2.zip, mirrored from 
thematicmapping.org." The country Polygons objects and the data slot data 
frame row numbers have been set to the ISO 3166 three letter codes.

All other files are copyright (c) 2001-2004 Roger S. Bivand and Nicholas
Lewin-Koh, and are subject to the licence at the foot of this file.

Licence
=======

    This is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.


The file COPYING in the R top-level directory is a copy of the "GNU
General Public License".

Roger.Bivand@nhh.no


